<?php
# WikiMedia Perforce Extension 
# 
# This extension adds support for linking and displaying Perforce
# objects such as changes, jobs, and file paths. It works in tandem
# with a browse mode P4Web to provide easy access to Perforce data.
# In the future it may include the ability to get formatted lists
# of jobs and changes.

#
# To install this plugin, add a section like the following to LocalSettings.php:
#
# require_once('extensions/Perforce/Perforce.php');
# $wgP4EXEC   = 'p4';              //path to the p4 client executable
# $wgP4PORT   = 'perforce:1666';   //Perforce server address:port
# $wgP4USER   = 'user';            //Perforce user with at least "list" access
# $wgP4PASSWD = 'password';        //Password/ticket for that user
# $wgP4WEBURL = "http://computer.perforce.com:8080/";
#

$wgExtensionFunctions[] = "perforceExtension";
$wgHooks['LanguageGetMagic'][] = 'wfPerforceLanguageGetMagic';

$wgExtensionCredits['parserhook'][] = array(
	'name' => 'Perforce',
	'url' => 'http://public.perforce.com/wiki/index.php/Perforce_%28MediaWiki_parser_hook%29',
	'author' => 'Matt Attaway',
	'description' => 'Display Perforce data in wiki pages',
);


function perforceExtension()
{
    global $wgParser;

    $wgParser->setHook( "changelist", "renderChangelist" );
    $wgParser->setHook( "recentchanges", 'tagP4Changes' );
    $wgParser->setFunctionHook( 'p4changes', 'runP4Changes' );
}

function wfPerforceLanguageGetMagic( &$magicWords, $langCode = "en" )
{
    switch( $langCode )
    {
    default:
         $magicWords['p4changes'] = array ( 0, 'p4changes' );
    }
    return true;    
}

# {{#p4changes:num|path|brief/long/full|user}}
function runP4Changes( &$parser, $num = '', $path = '', $desc = '', $user = '' )
{
    $argv['num']  = $num;
    $argv['path'] = $path;
    $argv['desc'] = $desc;
    $argv['user'] = $user;
    return renderRecentChanges( '', $argv, $parser, 'wiki' );
}

function tagP4Changes( $input, $argv, &$parser )
{
    return renderRecentChanges( $input, $argv, $parser, 'html' );
}

function renderChangelist( $input, $argv, &$parser ) 
{
    global $wgP4EXEC;
    global $wgP4PORT;
    global $wgP4USER;
    global $wgP4PASSWD;
    global $wgP4WEBURL;

    if( $argv["style"] )
    {
        $range = '@' . $input . ',' . $input;

        $cmdline = wfEscapeShellArg( $wgP4EXEC ) .
             " -u " . wfEscapeShellArgr( $wgP4USER ) .
             " -p " . wfEscapeShellArg( $wgP4PORT );

	if( $wgP4PASSWD != "" )
	    $cmdline .= " -P " . wfEscapeShellArg( $wgP4PASSWD );

        $cmdline .= " changes " . wfEscapeShellArg( $range );
        $text =  `{$cmdline}`;

        return formatShortChange( $text );
    }
    else
	return "<a href=\"$wgP4WEBURL" . $input . "?ac=10\">" . $input . "</a>";
}


function renderRecentchanges( $input, $argv, &$parser, $format )
{
    $parser->disableCache();

    global $wgP4EXEC;
    global $wgP4PORT;
    global $wgP4USER;
    global $wgP4PASSWD;

    $path = $argv["path"];
    $num  = $argv["num"];
    $desc = $argv["desc"];
    $user = $argv["user"];

    if( $path == "" || $num == "" )
	return "Please provide both a depot path(path attribute) and the number of changes to show(num attribute).";

    $arg = $path . '@' . $change; 

    $cmdline = $wgP4EXEC .
        " -u " . $wgP4USER .
        " -p " . $wgP4PORT ; 

    if( $wgP4PASSWD != "" )
        $cmdline .= " -P " . $wgP4PASSWD ;

    $cmdline .= " changes -m" . escapeshellarg( $num ) . " ";
    if ( $desc == "long" )
        $cmdline .= "-L ";
    if ( $desc == "full" )
        $cmdline .= "-l ";
    if ( $user != "" )
        $cmdline .= "-u " . escapeshellarg( $user ) . " ";
    $cmdline .= escapeshellarg( $path );
    $cmdline .= ' 2>&1';

    $changes = array();
    exec( $cmdline, $changes );

    $isDarkStripe = true;
    $output .= "<ul class=\"zebra\">";
    $inList = false;
    $inListList = false;

    foreach( $changes as $value )
    {
        if ( !preg_match( '/^Change [0-9]+ on/', $value ) )
        {
            if ( !$inListList )
            {
                $output .= '<ul>';
                $inListList = true;
            }
            $output .= $value;
            if ( $value )
                $output .= '<br/>';
            continue;
        }
        if ( $inListList )
        {
            $output .= '</ul>';
            $inListList = false;
        }
        if ( $inList )
        {
            $output .= "</li>\n";
            $inList = false;
        }

	if( $isDarkStripe )
	    $output .= "<li class=\"zebra\">";
	else
            $output .= "<li>";
        $inList = true;

        $output .= formatShortChange( $value, $format );

        $isDarkStripe = !$isDarkStripe;
    }
    if ( $inListList )
    {
        $output .= '</ul>';
        $inListList = false;
    }
    if ( $inList )
    {
        $output .= "</li>\n";
        $inList = false;
    }
    $output .= "</ul>";

    return $output;
}


function formatShortChange( $text, $format='html' )
{
    global $wgP4WEBURL;

    $pattern[0] = '/Change (\d+) /';
    $pattern[1] = '/ (\S+)@(\S+)/';

    switch( $format )
    {
    default:
    case 'html':
      $replacement[0] = 'Change <a href="' . $wgP4WEBURL . '${1}?ac=10\">${1}</a> ';
      $replacement[1] = ' <a href="' . $wgP4WEBURL . '${1}?ac=17">${1}</a>@<a href="' . $wgP4WEBURL . '${2}?ac=15">${2}</a> ';
      break;
    case 'wiki':
      $replacement[0] = 'Change [' . $wgP4WEBURL . '${1}?ac=10 ${1}] ';
      $replacement[1] = ' [' . $wgP4WEBURL . '${1}?ac=17 ${1}]@[' . $wgP4WEBURL . '${2}?ac=15 ${2}] ';
    break;
    }

    return preg_replace( $pattern, $replacement, $text );
}


?>
